<?php

namespace Drupal\image_sizes\Commands;

use Drupal\image\Entity\ImageStyle;
use Drupal\Core\Language\LanguageInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\image_sizes\Entity\ImageSizesPresetEntity;
use Symfony\Component\Console\Output\OutputInterface;
use Drush\Commands\DrushCommands;

/**
 * Class GenerateImageSizeCommand.
 */
class GenerateImageSizeCommand extends DrushCommands {

  /**
   * @var String
   */
  protected $ratio = FALSE;

  /**
   * @var array
   */
  protected $options = [];

  /**
   * @var boolean
   */
  protected $useFocalPoint = FALSE;

  /**
   * @var Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * @var Drupal\Core\Extension\ModuleHandle
   */
  protected $moduleHandler;

  /**
   * DeleteCommand constructor.
   *
   * @param EntityTypeManagerInterface    $entityTypeManager
   */
  public function __construct(
      EntityTypeManagerInterface $entityTypeManager,
      ModuleHandlerInterface $module_handler
  ) {
      $this->entityTypeManager = $entityTypeManager;
      $this->moduleHandler = $module_handler;
      parent::__construct();
  }

  /**
   * Generate image sizes presets.
   *
   * @param string $name
   *   Name.
   * @param string $min
   *   Min width.
   * @param string $max
   *   Max width.
   * @param string $steps
   *   Steps for each image style.
   * @command image-sizes:generate
   * @aliases isg
   * @option ratio
   *   Add a ratio.
   * @usage image-sizes:generate 4x3 50 1600 50 --ratio=4x3
   * @usage image-sizes:generate 4x3 50 1600 50 --ratio=4x3 --generate-thumbnail
   */
  public function generate($name, $min, $max, $steps, $options =
    [
      'ratio' => NULL,
      'use-focal-point' => FALSE,
      'generate-thumbnail' => NULL
    ]) {
    if ($options['use-focal-point'] && !$this->moduleHandler->moduleExists('focal_point')) {
      $this->io()->error('focal_point module is missing.');
      return;
    }
    if ($options['generate-thumbnail'] && !$this->moduleHandler->moduleExists('image_effects')) {
      $this->io()->error('image_effects module is missing.');
      return;
    }
    $this->options = [
      'name' => $name,
      'min' => $min,
      'max' => $max,
      'steps' => $steps,
      'options' => $options,
    ];
    $this->execute($this->output());
  }

  /**
   * {@inheritdoc}
   */
  protected function execute(OutputInterface $output) {
    $ratio = $this->options['options']['ratio'];
    $name = $this->options['name'];
    $min = $this->options['min'];
    $max = $this->options['max'];
    $steps = $this->options['steps'];
    $useFocalPoint = $this->options['options']['use-focal-point'];
    $ratios = ['x' => 0, 'y' => 0];
    $preloadStyle = $this->options['options']['generate-thumbnail'];
    $smallesStyle = FALSE;
    $ratios = FALSE;

    $style = ImageStyle::load($name);
    if ($style) {
      $output->writeln('Already exists');
      return;
    }

    if ($ratio != NULL) {
      $reg = '/^(\d+)x(\d+)$/';
      $matches = [];
      preg_match_all($reg, $ratio, $matches, PREG_SET_ORDER, 0);
      if (empty($matches)) {
        $output->writeln('Not correct ratio');
        return;
      }
      $ratios = ['x' => $matches[0][1], 'y' => $matches[0][2]];
    }

    $preset = ImageSizesPresetEntity::create([
      'label' => "$name",
      'id' => self::machineName($name),
    ]);

    for ($i = $min; $i <= $max; $i = $i + $steps ) {
      $style_name = $name . '_' . $i;
      $x = $i;

      $style = ImageStyle::create([
        'name' => $style_name,
        'label' => "$name $x"
      ]);
      $configuration = [
        'uuid' => NULL,
        'id' => ($useFocalPoint) ? 'focal_point_scale_and_crop' : 'image_scale_and_crop',
        'weight' => 0,
        'data' => [
          'width' => $x,
        ],
      ];
      if ($ratios !== FALSE) {
        $m = floor($i / $ratios['x']);
        $x = $ratios['x'] * $m;
        $y = $ratios['y'] * $m;
        $style = ImageStyle::create([
          'name' => $style_name,
          'label' => "${name} ${ratio} ${x}x${y}"]
        );
        $configuration = array(
          'uuid' => NULL,
          'id' => ($useFocalPoint) ? 'focal_point_scale_and_crop' : 'image_scale_and_crop',
          'weight' => 0,
          'data' => array(
            'width' => $x,
            'height' => $y,
          ),
        );
      }

      $effect = \Drupal::service('plugin.manager.image.effect')->createInstance($configuration['id'], $configuration);
      $style->addImageEffect($effect->getConfiguration());
      $style->save();
      $output->writeln("Generate Style ${style_name}");
      $preset->addStyle($style->id());
      if ($smallesStyle == FALSE) {
        $preset->setPreloadStyle($style->id());
        $smallesStyle = TRUE;
      }
    }
    if ($preloadStyle) {
      $x = 15;
      $style = ImageStyle::create([
        'name' => $name . '_preload',
        'label' => "$name Preload"
      ]);
      if ($ratios !== FALSE) {
        $m = floor($x / $ratios['x']);
        $x = $ratios['x'] * $m;
        $y = $ratios['y'] * $m;
        $configuration = array(
          'uuid' => NULL,
          'id' => ($useFocalPoint) ? 'focal_point_scale_and_crop' : 'image_scale_and_crop',
          'weight' => 0,
          'data' => array(
            'width' => $x,
            'height' => $y,
          ),
        );
      } else {
        $configuration = [
          'uuid' => NULL,
          'id' => ($useFocalPoint) ? 'focal_point_scale_and_crop' : 'image_scale_and_crop',
          'weight' => 0,
          'data' => [
            'width' => $x,
          ],
        ];
      }
      $effect = \Drupal::service('plugin.manager.image.effect')->createInstance($configuration['id'], $configuration);
      $style->addImageEffect($effect->getConfiguration());
      $effect = \Drupal::service('plugin.manager.image.effect')->createInstance('image_effects_gaussian_blur', [
        'uuid' => NULL,
        'id' => 'image_effects_gaussian_blur',
        'weight' => 10,
        'date' => [
          'radius' => 5,
          'sigma' => 2,
        ]
      ]);
      $style->addImageEffect($effect->getConfiguration());
      $style->save();
      $preset->setPreloadStyle($style->id());
    }
    $preset->setFallbackStyle($style->id());
    $preset->save();
  }

  public static function machineName($value) {
    $new_value = \Drupal::service('transliteration')->transliterate($value, LanguageInterface::LANGCODE_DEFAULT, '_');
    $new_value = strtolower($new_value);
    $new_value = preg_replace('/[^a-z0-9_]+/', '_', $new_value);
    return preg_replace('/_+/', '_', $new_value);
  }

}
